/* $Id$

This file is part of libmspfs
Copyright © 2006-2008  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#include <cerrno>
#include <msp/core/except.h>
#ifndef WIN32
#include <fnmatch.h>
#else
#include <msp/strings/glob.h>
#endif
#include <msp/strings/utils.h>
#include "dir.h"
#include "path.h"
#include "utils.h"

using namespace std;

namespace Msp {
namespace FS {

/**
Fixes the case of the given path to match files / directories on the
filesystem.  Intended to be used in programs that need to interact with
emulated Windows programs.
*/
Path fix_case(const Path &path)
{
	bool found=true;
	Path result;
	for(Path::Iterator i=path.begin(); i!=path.end(); ++i)
	{
		if(!found || *i=="/")
			result/=*i;
		else
		{
			list<string> files;
			if(result.size())
				files=list_files(result);
			else
				files=list_files(".");

			found=false;
			for(list<string>::iterator j=files.begin(); (j!=files.end() && !found); ++j)
				if(!strcasecmp(*j,*i))
				{
					result/=*j;
					found=true;
				}

			if(!found)
				result/=*i;
		}
	}

	return result;
}

void unlink(const Path &path)
{
	if(::unlink(path.str().c_str())==-1)
		throw SystemError("unlink failed", errno);
}

Filename splitext(const string &fn)
{
	Filename result;
	unsigned dot=fn.rfind('.');
	result.base=fn.substr(0, dot);
	if(dot!=string::npos)
		result.ext=fn.substr(dot);
	return result;
}

int fnmatch(const string &pat, const Path &fn)
{
#ifdef WIN32
	return globcasematch(pat, fn.str());
#else
	return ::fnmatch(pat.c_str(), fn.str().c_str(), FNM_PATHNAME);
#endif
}

Path relative(const Path &path, const Path &base)
{
	Path::Iterator i=path.begin();
	Path::Iterator j=base.begin();
	for(; (i!=path.end() && j!=base.end() && *i==*j); ++i, ++j) ;

	Path result;
	for(; j!=base.end(); ++j)
		result/="..";
	for(; i!=path.end(); ++i)
		result/=*i;

	return result;
}

/**
Extracts the basename from the given path.  Same thing as Path::Path(p)[-1],
but faster.
*/
string basename(const std::string &p)
{
	unsigned slash=p.rfind(DIRCHAR);
	if(slash==string::npos)
		return p;
	else
		return p.substr(slash+1);
}

} // namespace FS
} // namespace Msp
